#!/bin/bash

# Ensure the script is running with correct permissions
if [[ $(id -u) -ne 0 ]]; then
  echo "Please run this script as root or with sudo."
  exit 1
fi

# Configuration
CONFIG_FILE="node.conf"

# Check if the configuration file exists
if [[ ! -f "$CONFIG_FILE" ]]; then
  echo "Configuration file $CONFIG_FILE not found!"
  exit 1
fi

# Source the configuration file to set variables
source "$CONFIG_FILE"


# Define the total traffic (in MB) for the alerting thresholds
available_traffic=1024  # 1TB in GB (1 TB = 1024 GB)

# Set thresholds in MB for alerting
declare -A THRESHOLDS
THRESHOLDS[80]=80
THRESHOLDS[90]=90
THRESHOLDS[95]=95
THRESHOLDS[98]=98

# Monitoring and Backup
echo "Starting the node manager for node $NODE_HOSTNAME"

# Function to check domain IP
get_domain_ip() {
    dig +short "$NODE_DOMAIN" | tail -n1
}

# Function to check server IP
get_server_ip() {
    hostname -I | awk '{print $1}'
}

# Function to perform backup
perform_backup() {
    echo "$(date): Changes detected in $SOURCE_PATH. Backing up..." >> backup_log.txt
    sshpass -p "$SSH_PASSWORD" rsync -avz -e "ssh -p $SSH_PORT" "$SOURCE_PATH/" "$SSH_USER@$SSH_HOST:$REMOTE_PATH"
    echo "$(date): Backup completed." >> backup_log.txt
}

# Function to restore backup
restore_backup() {
    echo "$(date): Restoring backup from remote server..." >> backup_log.txt
    sshpass -p "$SSH_PASSWORD" rsync -avz -e "ssh -p $SSH_PORT" "$SSH_USER@$SSH_HOST:$REMOTE_PATH/" "$SOURCE_PATH"
    echo "$(date): Backup restored." >> backup_log.txt
    # Restart x-ui service
    x-ui restart
}

send_alert() {
    local message="$1"
    curl -s -X POST "https://api.telegram.org/bot$TELEGRAM_BOT_TOKEN/sendMessage" -d chat_id="$TELEGRAM_CHAT_ID" -d text="$message"
}

# Main loop
while true; do


	# Create the START_FILE if it doesn't exist
	START_FILE="traffic_start_point"
	if [ ! -f "$START_FILE" ]; then
		total_usage=$(vnstat --oneline | cut -d';' -f12)
		echo "$total_usage" | sudo tee "$START_FILE"
	fi

	# Read the starting point from the file
	start_usage=$(cat "$START_FILE")

	# Get the current total traffic usage in MB
	total_usage=$(vnstat --oneline | cut -d';' -f12)

	# Calculate the traffic used in the current period
	used_in_period=$((total_usage - start_usage))



	# Check and send alerts for each threshold
	for percent in "${!PERCENTAGES[@]}"; do
		threshold_mb=$((available_traffic * percent / 100))
		alert_file="traffic_alert_${percent}_sent"
		alert_message="Alert: Traffic usage has reached ${percent}% of the current ${available_traffic}GB block. Current usage: ${used_in_period}MB"

		if [ "$used_in_period" -ge "$threshold_mb" ] && [ ! -f "$alert_file" ]; then
			send_alert "$alert_message"
			sudo touch "$alert_file"
		fi
	done

	# Reset mechanism if usage exceeds 1TB
	if [ "$used_in_period" -ge "$available_traffic" ]; then
		echo "$total_usage" | sudo tee "$START_FILE"
		sudo rm -f "traffic_alert_"*"_sent"
	fi

    # Get the current IP of the domain
    DOMAIN_IP=$(get_domain_ip)
	SERVER_IP=$(get_server_ip)
	FLAG_FILE="ip_status.txt"
    # Compare the IPs and decide the next action
    if [[ "$DOMAIN_IP" != "$SERVER_IP" ]]; then
        # IP addresses do not match, record it in the flag file
        echo "$DOMAIN_IP" > "$FLAG_FILE"
        echo "$(date): Domain IP ($DOMAIN_IP) does not match server IP ($SERVER_IP). Skipping backup." >> backup_log.txt
		sleep "30"
    else
        # IP addresses match
        if [[ -f "$FLAG_FILE" ]]; then
            # Flag file exists, check its content
            RECORDED_IP=$(cat "$FLAG_FILE")
            if [[ "$RECORDED_IP" == "$SERVER_IP" ]]; then
				send_alert "Transition of Node $NODE_HOSTNAME from $RECORDED_IP to $SERVER_IP due to inactivity of $RECORDED_IP!"
                # The flag file indicates the IP change has been handled
                rm "$FLAG_FILE"
                restore_backup
            else
                echo "$(date): IP changed, but flag file indicates an old IP. Skipping restoration." >> backup_log.txt
            fi
        else
            # No flag file, proceed with backup
            perform_backup
        fi
		sleep "1"
    fi 
done
